@@ -183,34 +183,29 @@ func ClearAfter() string {
 }
 
 func FormatTitle(title string) string {
+	switch Shell {
 	// These shells don't support setting the console title.
-	if Shell == shell.ELVISH || Shell == shell.XONSH {
+	case shell.ELVISH, shell.XONSH, shell.TCSH:
 		return ""
-	}
+	case shell.BASH, shell.ZSH:
+		title = trimAnsi(title)
+		s := new(strings.Builder)
 
-	title = trimAnsi(title)
+		// We have to do this to prevent the shell from misidentifying escape sequences.
+		for _, char := range title {
+			escaped, shouldEscape := formats.EscapeSequences[char]
+			if shouldEscape {
+				s.WriteString(escaped)
+				continue
+			}
 
-	if Plain {
-		return title
-	}
-
-	if Shell != shell.BASH && Shell != shell.ZSH {
-		return fmt.Sprintf(formats.Title, title)
-	}
-
-	// We have to do this to prevent Bash/Zsh from misidentifying escape sequences.
-	s := new(strings.Builder)
-	for _, char := range title {
-		escaped, shouldEscape := formats.EscapeSequences[char]
-		if shouldEscape {
-			s.WriteString(escaped)
-			continue
+			s.WriteRune(char)
 		}
 
-		s.WriteRune(char)
+		return fmt.Sprintf(formats.Title, s.String())
+	default:
+		return fmt.Sprintf(formats.Title, trimAnsi(title))
 	}
-
-	return fmt.Sprintf(formats.Title, s.String())
 }
 
 func EscapeText(text string) string {
@@ -387,10 +382,6 @@ func writeEscapedAnsiString(text string) {
 	builder.WriteString(text)
 }
 
-func getAnsiFromColorString(colorString color.Ansi, isBackground bool) color.Ansi {
-	return Colors.ToAnsi(colorString, isBackground)
-}
-
 func write(s rune) {
 	if isInvisible {
 		return
@@ -433,17 +424,19 @@ func writeSegmentColors() {
 		return
 	}
 
-	if fg.IsTransparent() && len(BackgroundColor) != 0 { //nolint: gocritic
-		background := getAnsiFromColorString(BackgroundColor, false)
+	switch {
+	case fg.IsTransparent() && len(BackgroundColor) != 0:
+		background := Colors.ToAnsi(BackgroundColor, false)
 		writeEscapedAnsiString(fmt.Sprintf(colorise, background))
 		writeEscapedAnsiString(fmt.Sprintf(colorise, bg.ToForeground()))
-	} else if fg.IsTransparent() && !bg.IsEmpty() {
+	case fg.IsTransparent() && !bg.IsEmpty():
 		isTransparent = true
 		writeEscapedAnsiString(fmt.Sprintf(transparentStart, bg))
-	} else {
+	default:
 		if !bg.IsEmpty() && !bg.IsTransparent() {
 			writeEscapedAnsiString(fmt.Sprintf(colorise, bg))
 		}
+
 		if !fg.IsEmpty() {
 			writeEscapedAnsiString(fmt.Sprintf(colorise, fg))
 		}
@@ -499,7 +492,7 @@ func writeArchorOverride(match map[string]string, background color.Ansi, i int)
 	currentColor.Add(bg, fg)
 
 	if currentColor.Foreground().IsTransparent() && len(BackgroundColor) != 0 {
-		background := getAnsiFromColorString(BackgroundColor, false)
+		background := Colors.ToAnsi(BackgroundColor, false)
 		writeEscapedAnsiString(fmt.Sprintf(colorise, background))
 		writeEscapedAnsiString(fmt.Sprintf(colorise, currentColor.Background().ToForeground()))
 		return position
@@ -609,12 +602,20 @@ func asAnsiColors(background, foreground color.Ansi) (color.Ansi, color.Ansi) {
 	background = background.Resolve(CurrentColors, ParentColors)
 	foreground = foreground.Resolve(CurrentColors, ParentColors)
 
+	if bg, err := Colors.Resolve(background); err == nil {
+		background = bg
+	}
+
+	if fg, err := Colors.Resolve(foreground); err == nil {
+		foreground = fg
+	}
+
 	inverted := foreground == color.Transparent && len(background) != 0
 
-	backgroundAnsi := getAnsiFromColorString(background, !inverted)
-	foregroundAnsi := getAnsiFromColorString(foreground, false)
+	background = Colors.ToAnsi(background, !inverted)
+	foreground = Colors.ToAnsi(foreground, false)
 
-	return backgroundAnsi, foregroundAnsi
+	return background, foreground
 }
 
 func trimAnsi(text string) string {
