@@ -6,28 +6,29 @@ import (
 	"time"
 
 	"github.com/jandedobbeleer/oh-my-posh/src/build"
+	"github.com/jandedobbeleer/oh-my-posh/src/cache"
 	"github.com/jandedobbeleer/oh-my-posh/src/runtime"
 	"github.com/jandedobbeleer/oh-my-posh/src/runtime/http"
 )
 
 type release struct {
-	URL             string    `json:"url"`
-	HTMLURL         string    `json:"html_url"`
-	AssetsURL       string    `json:"assets_url"`
-	UploadURL       string    `json:"upload_url"`
+	CreatedAt       time.Time `json:"created_at"`
+	PublishedAt     time.Time `json:"published_at"`
+	NodeID          string    `json:"node_id"`
+	TagName         string    `json:"tag_name"`
 	TarballURL      string    `json:"tarball_url"`
 	ZipballURL      string    `json:"zipball_url"`
 	DiscussionURL   string    `json:"discussion_url"`
-	ID              int       `json:"id"`
-	NodeID          string    `json:"node_id"`
-	TagName         string    `json:"tag_name"`
+	HTMLURL         string    `json:"html_url"`
+	URL             string    `json:"url"`
+	UploadURL       string    `json:"upload_url"`
 	TargetCommitish string    `json:"target_commitish"`
 	Name            string    `json:"name"`
 	Body            string    `json:"body"`
-	Draft           bool      `json:"draft"`
+	AssetsURL       string    `json:"assets_url"`
+	ID              int       `json:"id"`
 	Prerelease      bool      `json:"prerelease"`
-	CreatedAt       time.Time `json:"created_at"`
-	PublishedAt     time.Time `json:"published_at"`
+	Draft           bool      `json:"draft"`
 }
 
 const (
@@ -36,9 +37,9 @@ const (
 
 	upgradeNotice = `
 A new release of Oh My Posh is available: %s → %s
-To upgrade, run: 'oh-my-posh upgrade'
+To upgrade, run: 'oh-my-posh upgrade%s'
 
-To enable automated upgrades, set 'auto_upgrade' to 'true' in your configuration.
+To enable automated upgrades, run: 'oh-my-posh enable upgrade'.
 `
 )
 
@@ -47,6 +48,7 @@ func Latest(env runtime.Environment) (string, error) {
 	if err != nil {
 		return "", err
 	}
+
 	var release release
 	// this can't fail
 	_ = json.Unmarshal(body, &release)
@@ -82,13 +84,17 @@ func Notice(env runtime.Environment, force bool) (string, bool) {
 		return "", false
 	}
 
-	oneWeek := 10080
-	env.Cache().Set(CACHEKEY, latest, oneWeek)
+	env.Cache().Set(CACHEKEY, latest, cache.ONEWEEK)
 
 	version := fmt.Sprintf("v%s", build.Version)
 	if latest == version {
 		return "", false
 	}
 
-	return fmt.Sprintf(upgradeNotice, version, latest), true
+	var forceUpdate string
+	if IsMajorUpgrade(version, latest) {
+		forceUpdate = " --force"
+	}
+
+	return fmt.Sprintf(upgradeNotice, version, latest, forceUpdate), true
 }
